﻿using System;
using System.Text;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Linq;
using System.Collections.Generic;
using System.Configuration;
using System.Net.NetworkInformation;
using System.Configuration;

namespace LocalNetworkInfo
{
    class Program
    {
        

        static void Main(string[] args)
        {
            List<IPAddress> ips = new List<IPAddress>();
            List<IPAddress> fromIp = new List<IPAddress>();
            List<IPAddress> toIp = new List<IPAddress>();
            List<IPAddress> suggestedIp = new List<IPAddress>();


            GetExcludeIpRange(ref fromIp, ref toIp);

            foreach (NetworkInterface netinterface in NetworkInterface.GetAllNetworkInterfaces())
            {
                

                Console.WriteLine(netinterface.Name);
                Console.WriteLine("  Operational Status ............ : {0}", netinterface.OperationalStatus.ToString());
                Console.WriteLine("  MAC ........................... : {0}", netinterface.GetPhysicalAddress());
                
                foreach (GatewayIPAddressInformation gatewayInfo in netinterface.GetIPProperties().GatewayAddresses)
                    Console.WriteLine("  Gateway ....................... : {0}", gatewayInfo.Address);

                foreach (UnicastIPAddressInformation unicastInfo in netinterface.GetIPProperties().UnicastAddresses)
                {
                    Console.WriteLine("  IP Addresses .................. : {0}", unicastInfo.Address);
                    Console.WriteLine("  Mask .......................... : {0}", unicastInfo.IPv4Mask);
                    if (unicastInfo.Address.ToString() != "127.0.0.1")
                        ips.Add(unicastInfo.Address);
                }
                Console.WriteLine();
            }

            Console.WriteLine();

            Console.WriteLine("  Outside IP (WAN) ............... : {0}", GetAssignedIp());

            foreach (IPAddress ip in ips)
            {
                for (int i = 0; i < fromIp.Count - 1;i++)
                {
                    if (fromIp[i].Address < ip.Address && ip.Address < toIp[i].Address)
                    {
                        if(!suggestedIp.Contains(ip))
                        {
                            suggestedIp.Add(ip); 
                        }
                    }
                }
                
            }

            Byte[] ipbytes;
            foreach (IPAddress ip in suggestedIp)
            {
                ipbytes = ip.GetAddressBytes();
                Console.WriteLine("  Suggested IP ................... : {0}.{1}.{2}.{3}", ipbytes[0], ipbytes[1], ipbytes[2], ipbytes[3] <= 10 ? ipbytes[3] + 10 : ipbytes[3] + 100);
            }


            Console.WriteLine();
            Console.WriteLine("Press any key to close this window.");
            Console.ReadKey();
            Console.WriteLine();

        }

        private static void GetExcludeIpRange(ref List<IPAddress> fromIp, ref List<IPAddress> toIp)
        {
            char[] charSeparator = new char[] { ',' };
            char[] rangeSeparator = new char[] { '-' };
            string configSetting = ConfigurationManager.AppSettings["excludeIpRange"];
            string[] ranges = configSetting.Split(charSeparator, StringSplitOptions.RemoveEmptyEntries);

            string[] toFrom;
            IPAddress ip;
            foreach (string range in ranges)
            {
                toFrom = range.Split(rangeSeparator);
                if (toFrom.Count() == 2)
                {
                    if(IPAddress.TryParse(toFrom[0].Trim(), out ip))
                    {
                        if (IPAddress.TryParse(toFrom[1].Trim(), out ip))
                        {
                            fromIp.Add(IPAddress.Parse(toFrom[0].Trim()));
                            toIp.Add(IPAddress.Parse(toFrom[1].Trim()));
                        }
                    }
                }
            }
        }

        private static string GetAssignedIp()
        {
            StreamReader reader;
            Stream dataStream;
            WebResponse response;
            string responseText;
            try
            {
                string url = "http://www.agaverobotics.com/spinneret/myip.aspx";
                Uri uri = new Uri(url);

                // Create a request using a URL that can receive a post.
                WebRequest request = WebRequest.Create(uri);

                // Set the Method property of the request to POST.
                request.Method = "GET";

                // Set the ContentType property of the WebRequest.
                request.ContentType = "application/x-www-form-urlencoded";

                request.UseDefaultCredentials = true;

                // Get the response.
                response = request.GetResponse();

                // Display the status.
                //Console.WriteLine(((HttpWebResponse)response).StatusDescription);

                // Get the stream containing content returned by the server.
                dataStream = response.GetResponseStream();

                // Open the stream using a StreamReader for easy access.
                reader = new StreamReader(dataStream);

                // Read the content.
                string responseFromServer = reader.ReadToEnd();

                // Display the content.
                responseText = responseFromServer;

                // Clean up the streams.
                reader.Close();
                dataStream.Close();
                response.Close();

                return responseText;
                
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
                return string.Empty;
            }          
            
        }

    }
}

//NetworkInterface[] adapters = NetworkInterface.GetAllNetworkInterfaces();
//foreach (NetworkInterface adapter in adapters)
//{
//    IPInterfaceProperties properties = adapter.GetIPProperties();
//    Console.WriteLine(adapter.Description);
//    Console.WriteLine("  DNS suffix ................... : {0}", properties.DnsSuffix);
//    Console.WriteLine("  DNS enabled .................. : {0}", properties.IsDnsEnabled);
//    Console.WriteLine("  Dynamically configured DNS ... : {0}", properties.IsDynamicDnsEnabled);
//}
//Console.WriteLine();
